#include <windows.h>
#include <stdio.h>
#include "w7Application.h"

#define BALL_TIMER 50
#define FIELD_WIDTH 1000
#define FIELD_MIN_HEIGHT 400
#define FIELD_MAX_HEIGHT 1500
#define STICK_SWAY_RATE 40

LONG WINAPI WndProc( HWND, UINT, WPARAM, LPARAM );

w7Application theApp;

int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow ) {
	static char szWndClassName[] = "w7pingpongWindowClass";
	WNDCLASS wc; HWND hwnd;
	MSG msg;

	wc.style = 0; //styl klasy
	wc.lpfnWndProc = (WNDPROC)WndProc; //adres procedury obsugi zdarze
	wc.cbClsExtra = 0; //liczba bajtw obszaru danych klasy
	wc.cbWndExtra = 0; //liczba bajtw obszaru danych okna
	wc.hInstance = hInstance; //uchwyt do instancji aplikacji
	wc.hIcon = LoadIcon( NULL, IDI_APPLICATION ); //uchwyt do ikony
	wc.hCursor = LoadCursor( NULL, IDC_ARROW ); //uchwyt do kursora
	wc.hbrBackground = (HBRUSH) COLOR_BACKGROUND+1; //kolor ta
	wc.lpszMenuName = NULL; //nazwa menu
	wc.lpszClassName = szWndClassName; //nazwa klasy

	RegisterClass( & wc );

	CreateMutex(NULL, true, "w7pingpongHostMutex");
	if(GetLastError() == ERROR_ALREADY_EXISTS) theApp.host = false;
	//theApp.host = false;

	hwnd = CreateWindow( szWndClassName, //nazwa WNDCLASS
		(theApp.host) ? "Host" : "Client", //tytu okna
			//WS_OVERLAPPEDWINDOW, //styl okna
			((theApp.host) ? WS_THICKFRAME | WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX : WS_BORDER | WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX) | CS_OWNDC,
			CW_USEDEFAULT, CW_USEDEFAULT, //pozycja
			FIELD_WIDTH, FIELD_MIN_HEIGHT,//rozmiar
			HWND_DESKTOP, //uchwyt do okna rodzica
			NULL, //uchwyt do menu
			hInstance, //uchwyt do instancji aplikacji
			NULL ); //dane (tworzenie okna)

	//GetClientRect( hwnd, &theApp.field );

	RECT field_cns, wrect;
	field_cns.top = theApp.field.top = 0;
	field_cns.bottom = theApp.field.bottom = FIELD_MAX_HEIGHT;
	field_cns.left = theApp.field.left = 0;
	field_cns.right = theApp.field.right = FIELD_WIDTH;
	field_cns = theApp.convertCoordsL2D(hwnd, field_cns);

	AdjustWindowRect( &field_cns, WS_TILEDWINDOW, false );
	GetWindowRect( hwnd, &wrect );

	theApp.wnd_width = field_cns.right-field_cns.left;
	theApp.wnd_max_height = field_cns.bottom-field_cns.top;
	theApp.wnd_min_height = theApp.wnd_max_height / 3;
	MoveWindow( hwnd, wrect.left, wrect.top, theApp.wnd_width, theApp.wnd_max_height, true);

	if(theApp.host) {
		SetTimer( hwnd, 1, BALL_TIMER, NULL );
	} else {
		theApp.alignStickToBottom();
		theApp.hostWnd = FindWindow(szWndClassName, "Host");
		PostMessage(theApp.hostWnd, WM_USER, (WPARAM)hwnd, 0);
	}

	ShowWindow( hwnd, nCmdShow );
	UpdateWindow( hwnd );

	//ptla obsugi wiadomoci
	while( GetMessage( & msg, NULL, 0, 0 ) ) { //pobranie wiadomoci z kolejki
		TranslateMessage( & msg ); //przeksztacenie wiadomoci (klawiatura)
		DispatchMessage( & msg ); //przekazanie wiad. do proc. obsugi zdarze
	}

	return msg.wParam;
}

LRESULT CALLBACK WndProc( HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
	PAINTSTRUCT ps;
	HDC hdc; RECT rect, wrect;

	switch( message ) {
		case WM_PAINT:
			hdc = BeginPaint( hwnd, & ps );
			
			SetMapMode( hdc, MM_LOMETRIC );
			theApp.drawStick( hdc );
			theApp.drawBall( hdc );
			
			//char tmp[256];
			//sprintf(tmp,"%d, %d", theApp.ball.x, theApp.ball.y);
			//DrawText( hdc, tmp, -1, & rect, DT_SINGLELINE | DT_CENTER | DT_VCENTER );*/
			//SetWindowText( hwnd, tmp );

			EndPaint( hwnd, & ps );
			return 0;
		case WM_CHAR:
			if(theApp.host) {
				switch(wParam) {
					case 'a':
						theApp.swayTheStick(-STICK_SWAY_RATE);
						break;
					case 'd':
						theApp.swayTheStick(STICK_SWAY_RATE);
						break;
					case 'r':
						theApp.ball.reset();
						break;
				}

				InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getStickInvRect()), true);
			}
			return 0;
		case WM_MOUSEMOVE:
			if(!theApp.host) {
				theApp.setStickPos((lParam & 0xFFFF)*6.0 - theApp.field.right);
				InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getStickInvRect()), true);
				
				PostMessage( theApp.hostWnd, WM_USER+3, theApp.stick.left, 0);
			}
			return 0;
		case WM_TIMER:
			theApp.swayTheBall();
			theApp.dispatchStickCollision();
			theApp.dispatchWallCollision();
			if(theApp.dispatchPoint()) {
				InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getStickInvRect()), true);
				if(theApp.clientWnd)
					PostMessage( theApp.clientWnd, WM_USER+4, theApp.local_score, theApp.remote_score);
			}
			InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getBallInvRect()), true);
			if(theApp.clientWnd /*&& theApp.ball.y + theApp.ball.size > theApp.field.bottom*/)
				PostMessage( theApp.clientWnd, WM_USER+1, theApp.ball.x, theApp.ball.y - theApp.field.bottom);
			return 0;
		case WM_SIZING:
			if( ((RECT*)lParam)->right - ((RECT*)lParam)->left != theApp.wnd_width )
				((RECT*)lParam)->right = ((RECT*)lParam)->left + theApp.wnd_width;
			if( ((RECT*)lParam)->bottom - ((RECT*)lParam)->top > theApp.wnd_max_height )
				((RECT*)lParam)->bottom = ((RECT*)lParam)->top + theApp.wnd_max_height;
			if( ((RECT*)lParam)->bottom - ((RECT*)lParam)->top < theApp.wnd_min_height )
				((RECT*)lParam)->bottom = ((RECT*)lParam)->top + theApp.wnd_min_height;
			GetClientRect( hwnd, &rect );
			theApp.field = theApp.convertCoordsD2L(hwnd, rect);
			//GetClientRect( hwnd, &theApp.field );
			if(theApp.clientWnd)
				PostMessage( theApp.clientWnd, WM_USER+2, theApp.field.right, theApp.field.bottom);
			break;
		case WM_USER: // client notifies host
			if(theApp.host) theApp.clientWnd = (HWND)wParam;
			break;
		case WM_USER+1: // ball position update
			theApp.setBallPos(wParam, lParam);
			InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getBallInvRect()), true);
			break;
		case WM_USER+2: // new dimensions from host
			rect.left = 0;
			rect.top = 0;
			rect.right = wParam;
			rect.bottom = lParam;

			theApp.field.right = rect.right;
			theApp.field.bottom = rect.bottom;
			theApp.alignStickToBottom();

			rect = theApp.convertCoordsL2D(hwnd, rect);
			AdjustWindowRect( &rect, WS_TILEDWINDOW, false );
			GetWindowRect( hwnd, &wrect );
			MoveWindow( hwnd, wrect.left, wrect.top, rect.right-rect.left, rect.bottom-rect.top, true);

			InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getStickInvRect()), true);
			break;
		case WM_USER+3: // client's stick coordinates
			theApp.remote_stick.left = wParam;
			break;
		case WM_USER+4: // points update from host
			theApp.remote_score = wParam;
			theApp.local_score = lParam;
			InvalidateRect( hwnd, &theApp.convertCoordsL2D(hwnd, theApp.getStickInvRect()), true);
			break;
    case WM_DESTROY:
			PostQuitMessage( 0 ); //wysanie wiadomoci WM_QUIT
		return 0;
	}

	return DefWindowProc( hwnd, message, wParam, lParam );
}