#include <windows.h>
#include <math.h>
#include <stdio.h>
#include "w7Application.h"

#ifndef M_PI
#define M_PI 3.14159265
#endif

#define DEF_BALL_SPEED 20
#define STICK_WIDTH 400
#define STICK_THICKNESS 50
#define HITPOINT_DIVIDER 2

int paraSgn(int n) {
	return (n < 0) ? -1 : 1;
}

w7Ball::w7Ball() {
	reset();
}

void w7Ball::reset() {
	x = 200;
	y = 500;
	size = 40;
	rate = DEF_BALL_SPEED;
	xrate = 0;
	yrate = -DEF_BALL_SPEED;
}

// =====================================

w7Application::w7Application() {
	stick.left = remote_stick.left = 0;
	stick.right = STICK_WIDTH;
	stick.top = 0;
	stick.bottom = STICK_THICKNESS;
	host = true;
	clientWnd = 0;
	local_score = 0;
	remote_score = 0;
}

bool w7Application::dispatchPoint(){
	if( ball.y < stick.top ) {
		remote_score++;
		ball.reset();
		return true;
	} else if( ball.y > 2*field.bottom ) {
		local_score++;
		ball.reset();
		return true;
	}
	return false;
}

void w7Application::dispatchStickCollision() {
	if( ((ball.x+ball.size > stick.left) && (ball.x < stick.left + STICK_WIDTH) &&
	    (ball.y > stick.top) && (ball.y < stick.top + STICK_THICKNESS)) || 

			((ball.x+ball.size > remote_stick.left) && (ball.x < remote_stick.left + STICK_WIDTH) &&
			(ball.y+ball.size > 2*field.bottom - STICK_THICKNESS) && (ball.y+ball.size < 2*field.bottom))) {

		float hitpoint;

		if(paraSgn(ball.yrate) > 0) {
			//ball.y = remote_stick.top-1;
			ball.y -= ball.y+ball.size-2*field.bottom+STICK_THICKNESS+1;
			hitpoint = -(remote_stick.left + STICK_WIDTH/2.0 - ball.x - ball.size/2.0) / (STICK_WIDTH/2.0 + ball.size) / HITPOINT_DIVIDER;
		} else {
			//ball.y = stick.bottom+1;
			ball.y += STICK_THICKNESS - ball.y + 1;
			hitpoint = -(stick.left + STICK_WIDTH/2.0 - ball.x - ball.size/2.0) / (STICK_WIDTH/2.0 + ball.size) / HITPOINT_DIVIDER;
		}

		ball.xrate = sin(M_PI*hitpoint) * ball.rate;
		ball.yrate = cos(M_PI*hitpoint) * ball.rate * -paraSgn(ball.yrate);
		ball.x += ball.xrate;
	}
}

void w7Application::dispatchWallCollision() {
	if( ball.x < 0 || ball.x+ball.size > field.right)
		ball.xrate = -ball.xrate;
}

void w7Application::swayTheStick(int x_) {
	old_stick = stick;
	stick.left += x_;
	if(stick.left >= field.right) stick.left = field.right;
	if(stick.left <= -STICK_WIDTH) stick.left = -STICK_WIDTH;
	stick.right = stick.left + STICK_WIDTH;
}

void w7Application::setStickPos(int x_) {
	old_stick = stick;
	stick.left = x_;
	if(stick.left >= field.right) stick.left = field.right;
	if(stick.left <= -STICK_WIDTH) stick.left = -STICK_WIDTH;
	stick.right = stick.left + STICK_WIDTH;
}

void w7Application::swayTheBall() {
	old_ball = ball;
	ball.x += ball.xrate;
	ball.y += ball.yrate;
}

void w7Application::setBallPos(int x_, int y_) {
	old_ball = ball;
	ball.x = x_;
	ball.y = y_;
}

RECT w7Application::getStickInvRect() {
	RECT tmp;
	tmp.left = (stick.left < old_stick.left) ? stick.left : old_stick.left;
	tmp.right = (stick.right > old_stick.right) ? stick.right : old_stick.right;
	tmp.top = (stick.top < old_stick.top) ? stick.top : old_stick.top;
	tmp.bottom = (stick.bottom > old_stick.bottom) ? stick.bottom : old_stick.bottom;
	return tmp;
}

RECT w7Application::getBallInvRect() {
	RECT tmp;
	if(ball.x < old_ball.x) {
		tmp.left = ball.x;
		tmp.right = old_ball.x + old_ball.size;
	} else {
		tmp.left = old_ball.x;
		tmp.right = ball.x + ball.size;
	}

	if(ball.y < old_ball.y) {
		tmp.top = ball.y;
		tmp.bottom = old_ball.y + old_ball.size;
	} else {
		tmp.top = old_ball.y;
		tmp.bottom = ball.y + ball.size;
	}

	tmp.top--;
	tmp.bottom++;
	tmp.left--;
	tmp.right++;
	return tmp;
}

void w7Application::alignStickToBottom() {
	old_stick = stick;
	stick.top = field.bottom - STICK_THICKNESS;
	stick.bottom = field.bottom;
}

int w7Application::drawStick(HDC hdc){
	RECT tmp = stick;
	tmp.top = -stick.top;
	tmp.bottom = -stick.bottom;
	FillRect( hdc, &tmp, (HBRUSH)GetStockObject(BLACK_BRUSH) );
	char tmpstr[256];
	sprintf(tmpstr, "%d, %d", local_score, remote_score);
	return DrawText( hdc, tmpstr, -1, &tmp, DT_SINGLELINE | DT_CENTER | DT_VCENTER );
}

int w7Application::drawBall(HDC hdc){
	//return Arc( hdc, ball.x, ball.y, ball.x+ball.size, ball.y+ball.size, ball.x, ball.y, ball.x, ball.y);
	return Arc( hdc, ball.x, -ball.y, ball.x+ball.size, -(ball.y+ball.size), ball.x, ball.y, ball.x, ball.y);
}

RECT w7Application::convertCoordsL2D(HWND hwnd, RECT tmp){
	HDC hdc = GetDC(hwnd);
	SetMapMode( hdc, MM_LOMETRIC );
	POINT p[2];
	p[0].x = tmp.left;
	p[0].y = -tmp.top;
	p[1].x = tmp.right;
	p[1].y = -tmp.bottom;
	LPtoDP(hdc, p, 2);
	tmp.left = p[0].x;
	tmp.top = p[0].y;
	tmp.right = p[1].x;
	tmp.bottom = p[1].y;
	ReleaseDC(hwnd, hdc);
	return tmp;
}

RECT w7Application::convertCoordsD2L(HWND hwnd, RECT tmp){
	HDC hdc = GetDC(hwnd);
	SetMapMode( hdc, MM_LOMETRIC );
	POINT p[2];
	p[0].x = tmp.left;
	p[0].y = -tmp.top;
	p[1].x = tmp.right;
	p[1].y = -tmp.bottom;
	DPtoLP(hdc, p, 2);
	tmp.left = p[0].x;
	tmp.top = p[0].y;
	tmp.right = p[1].x;
	tmp.bottom = p[1].y;
	ReleaseDC(hwnd, hdc);
	return tmp;
}